#!/bin/bash

HTTPCONF="/etc/httpd"
BINDZONEPATH="/var/named"
BINDCONF="/etc/named.conf"
HTTPROOT="/var/www/html"
SERIAL=$(date +%Y%m%d01)
POSTPATH="/etc/postfix"
PASS="$(date +%s | sha256sum | base64 | head -c 8 ; echo)"

if [ "$EUID" -ne 0 ]
then echo "Please run as root"
exit
fi

version=$(cat /etc/os-release |grep -i "VERSION" | head -1 |tr -dc '0-9.'|cut -d \. -f1)
distro=$(cat /etc/os-release | grep "PRETTY_NAME" | sed 's/PRETTY_NAME=//g' | sed 's/["]//g' | awk '{print $1}')
if [ "$distro" == "CentOS" ]
then
echo "CentOS $version detected"
if [ "$version" -lt "7" ]
then
"Version 7 or 8 needed. terminating"
exit 1
fi
else
echo "Not CentOS Linux, terminated"
exit 1
fi

if [ "$#" -ne 2 ]; then
echo "Incorrect parameters, Use: ./installserver.sh <domain.com> <external.ip.address>"
exit 1
else
domain=$1
ipaddr=$2
ptr=$(echo $ipaddr | awk -F. -vOFS=. '{print $3,$2,$1,"in-addr.arpa"}')
ptrlast=$(echo $ipaddr | awk -F. -vOFS=. '{print $4}')

echo "Disabling SE Linux"
if setenforce 0 > /dev/null 2>&1 
then
setenforce 0
sed -i "s/SELINUX=enforcing/SELINUX=disabled/g" /etc/sysconfig/selinux
fi

echo "Domain: $domain"
  
echo "Setting ServerName to server.$domain"
hostnamectl set-hostname server.$domain
echo "$ipaddr server.$domain" >> /etc/hosts

echo "Updating system"
yum -y install epel-release > /dev/null
sleep 1

echo "Installing Wget"
yum -y install wget > /dev/null

echo "Installing OpenSSL"
yum -y install openssl > /dev/null
  
echo "Installing Bind"
yum -y install bind bind-utils > /dev/null

echo "Extracting files"
tar xvf files.tar > /dev/null

echo "Installing Firewall"
yum -y install firewalld > /dev/null
systemctl enable firewalld > /dev/null
systemctl start firewalld > /dev/null
firewall-cmd --permanent --zone=public --add-port=53/tcp > /dev/null
firewall-cmd --permanent --zone=public --add-port=80/tcp > /dev/null
firewall-cmd --permanent --zone=public --add-port=443/tcp > /dev/null
firewall-cmd --permanent --zone=public --add-port=587/tcp > /dev/null
firewall-cmd --permanent --zone=public --add-port=995/tcp > /dev/null
firewall-cmd --permanent --zone=public --add-port=993/tcp > /dev/null
firewall-cmd --reload > /dev/null
echo "Required Ports Opened"

echo "Configure DNS Server for zone $domain"
cp $BINDCONF $BINDCONF.res
echo "zone \"$domain\" IN { " >>  $BINDCONF
echo "    type master;"  >>  $BINDCONF
echo "    file \"$domain.db\";" >>  $BINDCONF
echo "};"  >>  $BINDCONF
echo ""
echo "zone \"$ptr\" IN { " >>  $BINDCONF
echo "    type master;"  >>  $BINDCONF
echo "    file \"$domain-ptr.db\";" >>  $BINDCONF
echo "};"  >>  $BINDCONF
echo ""

echo "Creating SOA zone file for $domain"
if [ -f $BINDZONEPATH/$domain.db ]
then
rm -f $BINDZONEPATH/$domain.db
fi

touch $BINDZONEPATH/$domain.db
echo "\$TTL 120" >> $BINDZONEPATH/$domain.db
echo "@	IN	SOA	ns1.$domain.	postmaster.$domain. (" >> $BINDZONEPATH/$domain.db
echo "			$SERIAL  ; Serial" >> $BINDZONEPATH/$domain.db
echo "		    604800 ; refresh" >> $BINDZONEPATH/$domain.db
echo "		     86400 ; retry" >> $BINDZONEPATH/$domain.db
echo "		   2419200 ; expiration" >> $BINDZONEPATH/$domain.db
echo "		    604800 ; TTL negative cache" >> $BINDZONEPATH/$domain.db
echo "  ); " >> $BINDZONEPATH/$domain.db
echo " " >> $BINDZONEPATH/$domain.db
echo "@	IN	NS	ns1.$domain." >> $BINDZONEPATH/$domain.db
echo "@	IN	NS	ns2.$domain." >> $BINDZONEPATH/$domain.db
echo "@       IN      MX      10       server.$domain." >> $BINDZONEPATH/$domain.db
echo "@	IN	A       $ipaddr " >> $BINDZONEPATH/$domain.db
echo "ns1	IN	A	$ipaddr " >> $BINDZONEPATH/$domain.db
echo "ns2     IN      A       $ipaddr " >> $BINDZONEPATH/$domain.db
echo "server  IN     A       $ipaddr " >> $BINDZONEPATH/$domain.db
echo "www	 IN	A	$ipaddr " >> $BINDZONEPATH/$domain.db

echo "Creating PTR zone file for $domain"
if [ -f $BINDZONEPATH/$domain-ptr.db ]
then
rm -f $BINDZONEPATH/$domain-ptr.db
fi
touch $BINDZONEPATH/$domain-ptr.db
echo "\$TTL 3H" >> $BINDZONEPATH/$domain-ptr.db
echo "@       IN      SOA     ns1.$domain.        postmaster.$domain. (" >> $BINDZONEPATH/$domain-ptr.db
echo "                       $SERIAL  ; Serial" >> $BINDZONEPATH/$domain-ptr.db
echo "                    604800 ; refresh" >> $BINDZONEPATH/$domain-ptr.db
echo "                     86400 ; retry" >> $BINDZONEPATH/$domain-ptr.db
echo "                   2419200 ; expiration" >> $BINDZONEPATH/$domain-ptr.db
echo "                    604800 ; TTL negative cache" >> $BINDZONEPATH/$domain-ptr.db
echo "  ); " >> $BINDZONEPATH/$domain-ptr.db
echo " " >> $BINDZONEPATH/$domain-ptr.db
echo "        NS      ns1.$domain." >> $BINDZONEPATH/$domain-ptr.db
echo "        NS      ns2.$domain." >> $BINDZONEPATH/$domain-ptr.db
echo " " >> $BINDZONEPATH/$domain-ptr.db
echo "$ptrlast	IN	PTR $domain." >> $BINDZONEPATH/$domain-ptr.db
sed -i "s/127.0.0.1/any/g" /etc/named.conf
sed -i "s/listen-on-v6/#listen-on-v6/g"  /etc/named.conf
sed -i "s/localhost;/any;/g" /etc/named.conf 

echo "Enabling services and creating firewall rules"
echo "Checking if firewall is enabled"
systemctl start firewalld
systemctl is-active --quiet firewalld
if [ "$?" == "0" ]
then
    echo "Adding 80/tcp port"
    firewall-cmd --permanent --add-service=http
    echo "Adding 443/tcp port"
    firewall-cmd --permanent --add-port=443/tcp
    echo "Adding 995/tcp port"
    firewall-cmd --permanent --add-port=995/tcp
    echo "Adding 587/tcp port"
    firewall-cmd --permanent --add-port=587/tcp
    echo "Adding 993/tcp port"
    firewall-cmd --permanent --add-port=993/tcp
    echo "Adding DNS ports"
    firewall-cmd --permanent --zone=public --add-service=dns
    firewall-cmd --reload
fi

echo "Enabling and starting Bind"
systemctl enable named > /dev/null
chown -R named:named $BINDZONEPATH/* > /dev/null
service named start > /dev/null
  
echo "Installing Apache Server"
yum -y install mod_ssl openssl httpd  > /dev/null
systemctl enable httpd > /dev/null

echo "Installing PHP"
yum -y install http://rpms.remirepo.net/enterprise/remi-release-7.rpm > /dev/null
yum -y install epel-release yum-utils > /dev/null
yum-config-manager --disable remi-php54 > /dev/null
yum-config-manager --enable remi-php74 > /dev/null
yum -y install php php-intl php-zip php-mysql php-mbstring php-common php php-common php-curl php-json php-xml  php-cli php-gd > /dev/null
service httpd stop > /dev/null
   
echo "Installing LetsEncrypt"
yum install -y certbot-apache  > /dev/null
certbot  certonly --standalone -d server.$domain -d $domain -d www.$domain --preferred-challenges http --agree-tos -n -m postmaster@$domain --keep-until-expiring
  
echo "Enabling auto-renew certified on crontab"
echo "0 0 */10 * * certbot renew >> /var/logs/certbot-cron.log 2>&1" >> /var/spool/cron/root
echo "5 0 */10 * * service postfix restart ; service httpd restart"  >> /var/spool/cron/root
service crond restart

echo "Setting TimeZone to: US/Central in php.ini"
sed -i "s/;date.timezone =/date.timezone = US\/Central/g" /etc/php.ini
cp /etc/httpd/conf.d/ssl.conf /etc/httpd/conf.d/ssl.conf.res
sed -i "s/Listen 443/Listen 0.0.0.0:443/g" /etc/httpd/conf.d/ssl.conf
lineNum="$(grep -n "ServerName"  /etc/httpd/conf/httpd.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/httpd/conf/httpd.conf
sed -i "${lineNum}i ServerName www.$domain" /etc/httpd/conf/httpd.conf
lineNum="$(grep -n "DocumentRoot \""  /etc/httpd/conf/httpd.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/httpd/conf/httpd.conf
sed -i "${lineNum}i DocumentRoot $HTTPROOT/www.$domain" /etc/httpd/conf/httpd.conf

echo "Creating Apache Virtual Host"
mkdir -p $HTTPCONF/sites-enabled
mkdir -p $HTTPCONF/sites-available
lineNum="$(grep -n "ServerName"  /etc/httpd/conf.d/ssl.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/httpd/conf.d/ssl.conf
sed -i "${lineNum}i ServerName www.$domain" /etc/httpd/conf.d/ssl.conf
lineNum="$(grep -n "SSLCipherSuite"  /etc/httpd/conf.d/ssl.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/httpd/conf.d/ssl.conf
sed -i "${lineNum}i SSLCipherSuite \"EECDH+ECDSA+AESGCM EECDH+aRSA+AESGCM EECDH+ECDSA+SHA384 EECDH+ECDSA+SHA256 EECDH+aRSA+SHA384 EECDH+aRSA+SHA256 EECDH+aRSA+RC4 EECDH EDH+aRSA \!RC4 \!aNULL \!eNULL \!LOW \!3DES \!MD5 \!EXP \!PSK \!SRP \!DSS\"" /etc/httpd/conf.d/ssl.conf
lineNum="$(grep -n "SSLCertificateFile /etc/pki"  /etc/httpd/conf.d/ssl.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/httpd/conf.d/ssl.conf
sed -i "${lineNum}i SSLCertificateFile /etc/letsencrypt/live/server.$domain/cert.pem" /etc/httpd/conf.d/ssl.conf
lineNum="$(grep -n "SSLCertificateKeyFile /etc/pki"  /etc/httpd/conf.d/ssl.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/httpd/conf.d/ssl.conf
sed -i "${lineNum}i SSLCertificateKeyFile /etc/letsencrypt/live/server.$domain/privkey.pem" /etc/httpd/conf.d/ssl.conf
lineNum="$(grep -n "SSLCertificateChainFile /etc/pki"  /etc/httpd/conf.d/ssl.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/httpd/conf.d/ssl.conf
sed -i "${lineNum}i SSLCertificateChainFile /etc/letsencrypt/live/server.$domain/chain.pem" /etc/httpd/conf.d/ssl.conf    
echo "IncludeOptional sites-enabled/*.conf" >> $HTTPCONF/conf/httpd.conf

dm="server.$domain www.$domain"
for i in $dm
do  
mkdir -p $HTTPROOT/$i 
echo "<VirtualHost *:443>" >> $HTTPCONF/sites-available/$i.conf
echo "SSLEngine on" >> $HTTPCONF/sites-available/$i.conf
echo "SSLCertificateFile  /etc/letsencrypt/live/server.$domain/cert.pem" >> $HTTPCONF/sites-available/$i.conf
echo "SSLCertificateKeyFile /etc/letsencrypt/live/server.$domain/privkey.pem" >> $HTTPCONF/sites-available/$i.conf
echo "ServerAdmin webmaster@$domain" >> $HTTPCONF/sites-available/$i.conf
echo "ServerName $i"  >> $HTTPCONF/sites-available/$i.conf
echo "DocumentRoot $HTTPROOT/$i/" >> $HTTPCONF/sites-available/$i.conf
echo "ErrorLog /var/log/httpd/$i.error.log" >> $HTTPCONF/sites-available/$i.conf
echo "LogLevel warn" >> $HTTPCONF/sites-available/$i.conf
echo "CustomLog /var/log/httpd/$i.access.log combined" >> $HTTPCONF/sites-available/$i.conf
echo "</VirtualHost>" >> $HTTPCONF/sites-available/$i.conf

echo "Enabling domain $i"
ln -s $HTTPCONF/sites-available/$i.conf $HTTPCONF/sites-enabled/$i.conf
chown -R apache:apache $HTTPROOT/$i
done
lineNum=$(grep -n "ServerName"  $HTTPCONF/sites-available/www.$domain.conf | head -n 1 | cut -d: -f1)
sed -i "${lineNum}i ServerAlias $domain" $HTTPCONF/sites-available/www.$domain.conf

echo "Enable redirect all HTTP traffic to HTTPS"
echo "RewriteEngine On" >> /etc/httpd/conf/httpd.conf
echo "RewriteRule ^ - [E=protossl]" >> /etc/httpd/conf/httpd.conf
echo "RewriteCond %{HTTPS} on" >> /etc/httpd/conf/httpd.conf
echo "RewriteRule ^ - [E=protossl:s]" >> /etc/httpd/conf/httpd.conf
echo "RewriteCond %{HTTPS} !=on" >> /etc/httpd/conf/httpd.conf
echo "RewriteRule ^ https://%{HTTP_HOST}%{REQUEST_URI} [L,R=301]" >> /etc/httpd/conf/httpd.conf  
service httpd start 

echo "Installing Postfix, DKIM, DMARC and SPF"
useradd -r -u 150 -g mail -d /var/vmail -s /sbin/nologin -c "Virtual Mail User" vmail
mkdir -p /var/vmail
chmod -R 770 /var/vmail
chown -R vmail:mail /var/vmail
yum -y install postfix opendkim pypolicyd-spf   perl-Getopt-Long postfix-mysql > /dev/null
adduser policyd-spf --user-group --no-create-home -s /bin/false

echo "Configuring SPF"
echo "policyd-spf unix - n n - 0 spawn user=policyd-spf argv=/usr/libexec/postfix/policyd-spf"  >> /etc/postfix/master.cf

echo "Configuring DKIM"
mkdir -p /etc/opendkim/keys/$domain
opendkim-genkey -b 2048 -d $domain -D /etc/opendkim/keys/$domain -s default -v > /dev/null
chown -R opendkim:opendkim /etc/opendkim
chmod go-rw /etc/opendkim/keys
lineNum="$(grep -n "Mode"  /etc/opendkim.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/opendkim.conf
sed -i "${lineNum}i Mode sv" /etc/opendkim.conf
lineNum="$(grep -n "SendReports"  /etc/opendkim.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/opendkim.conf
sed -i "${lineNum}i SendReports no" /etc/opendkim.conf
sed -i "s/# KeyTable/KeyTable/g" /etc/opendkim.conf
sed -i "s/# SigningTable/SigningTable/g" /etc/opendkim.conf
sed -i "s/# ExternalIgnoreList/ExternalIgnoreList/g" /etc/opendkim.conf
sed -i "s/# InternalHosts/InternalHosts/g" /etc/opendkim.conf
echo "*@$domain default._domainkey.$domain" >> /etc/opendkim/SigningTable
echo "default._domainkey.$domain $domain:default:/etc/opendkim/keys/$domain/default.private" >> /etc/opendkim/KeyTable 
echo "*.$domain" >> /etc/opendkim/TrustedHosts
DKIM=$(cat /etc/opendkim/keys/$domain/default.txt)
  
echo "Adding DKIM to Postfix"
echo "milter_default_action = accept" >> /etc/postfix/main.cf
echo "milter_protocol = 6 " >> /etc/postfix/main.cf
echo "smtpd_milters = inet:127.0.0.1:8891 " >> /etc/postfix/main.cf
echo "non_smtpd_milters = $smtpd_milters" >> /etc/postfix/main.cf
gpasswd -a postfix opendkim
systemctl enable postfix
systemctl enable opendkim
systemctl start postfix
systemctl start opendkim
 
echo "Adding DKIM,SPF and DMARC to DNS records"
SERIAL=$(date +%Y%m%d05)
lineNum="$(grep -n "Serial" /var/named/$domain.db | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /var/named/$domain.db
sed -i "${lineNum}i                  $SERIAL ; Serial" /var/named/$domain.db
echo "@      IN      TXT  \"v=spf1 mx ip4:$ipaddr ~all\"" >> /var/named/$domain.db
echo "_dmarc IN      TXT \"v=DMARC1; p=none; pct=100; rua=mailto:postmaster@$domain\"" >> /var/named/$domain.db
echo "$DKIM" >> /var/named/$domain.db
service named restart

echo "Installing MariaDB"
cp -f files/MariaDB.repo /etc/yum.repos.d/MariaDB.repo
echo "baseurl = http://yum.mariadb.org/10.5/centos$version-amd64" >> /etc/yum.repos.d/MariaDB.repo
yum -y install MariaDB-server MariaDB-backup > /dev/null
systemctl enable mariadb.service > /dev/null
systemctl start mariadb.service > /dev/null
  
echo "Creating database Postfixadmin"
mysql -u root --password="" -e "CREATE DATABASE postfixadmin; GRANT ALL PRIVILEGES ON postfixadmin.* TO 'postfixadmin'@'localhost' IDENTIFIED BY '${PASS}'; FLUSH PRIVILEGES;"
mysql -u root --password="" postfixadmin < files/adminmail.sql
service httpd restart > /dev/null

echo "Installing Postfixadmin database over postfix"
mkdir -p /etc/postfix/sql/
echo "user = postfixadmin" >> $POSTPATH/sql/mysql_virtual_alias_domain_catchall_maps.cf  
echo "password = $PASS" >> $POSTPATH/sql/mysql_virtual_alias_domain_catchall_maps.cf
echo "hosts = localhost" >> $POSTPATH/sql/mysql_virtual_alias_domain_catchall_maps.cf
echo "dbname = postfixadmin"  >> $POSTPATH/sql/mysql_virtual_alias_domain_catchall_maps.cf
echo "query = SELECT goto FROM alias,alias_domain WHERE alias_domain.alias_domain = '%d' and alias.address = CONCAT('@', alias_domain.target_domain) AND alias.active = 1 AND alias_domain.active='1'" >> $POSTPATH/sql/mysql_virtual_alias_domain_catchall_maps.cf
echo "user = postfixadmin" >> $POSTPATH/sql/mysql_virtual_alias_domain_mailbox_maps.cf
echo "password = $PASS" >> $POSTPATH/sql/mysql_virtual_alias_domain_mailbox_maps.cf
echo "hosts = localhost " >> $POSTPATH/sql/mysql_virtual_alias_domain_mailbox_maps.cf
echo "dbname = postfixadmin" >> $POSTPATH/sql/mysql_virtual_alias_domain_mailbox_maps.cf
echo "query = SELECT maildir FROM mailbox,alias_domain WHERE alias_domain.alias_domain = '%d' and mailbox.username = CONCAT('%u', '@', alias_domain.target_domain) AND mailbox.active = 1 AND alias_domain.active='1'" >> $POSTPATH/sql/mysql_virtual_alias_domain_mailbox_maps.cf
echo "user = postfixadmin" >> $POSTPATH/sql/mysql_virtual_alias_domain_maps.cf
echo "password = $PASS" >> $POSTPATH/sql/mysql_virtual_alias_domain_maps.cf
echo "hosts = localhost" >> $POSTPATH/sql/mysql_virtual_alias_domain_maps.cf
echo "dbname = postfixadmin" >> $POSTPATH/sql/mysql_virtual_alias_domain_maps.cf
echo "query = SELECT goto FROM alias,alias_domain WHERE alias_domain.alias_domain = '%d' and alias.address = CONCAT('%u', '@', alias_domain.target_domain) AND alias.active = 1 AND alias_domain.active='1'" >> $POSTPATH/sql/mysql_virtual_alias_domain_maps.cf
echo "user = postfixadmin"  >> $POSTPATH/sql/mysql_virtual_alias_maps.cf
echo "password = $PASS" >> $POSTPATH/sql/mysql_virtual_alias_maps.cf
echo "hosts = localhost" >> $POSTPATH/sql/mysql_virtual_alias_maps.cf
echo "dbname = postfixadmin" >> $POSTPATH/sql/mysql_virtual_alias_maps.cf
echo "query = SELECT goto FROM alias WHERE address='%s' AND active = '1'" >> $POSTPATH/sql/mysql_virtual_alias_maps.cf
echo "user = postfixadmin" >> $POSTPATH/sql/mysql_virtual_domains_maps.cf
echo "password = $PASS" >> $POSTPATH/sql/mysql_virtual_domains_maps.cf
echo "hosts = localhost" >> $POSTPATH/sql/mysql_virtual_domains_maps.cf
echo "dbname = postfixadmin" >> $POSTPATH/sql/mysql_virtual_domains_maps.cf
echo "query = SELECT domain FROM domain WHERE domain='%s' AND active = '1'" >> $POSTPATH/sql/mysql_virtual_domains_maps.cf
echo "user = postfixadmin" >> $POSTPATH/sql/mysql_virtual_mailbox_limit_maps.cf
echo "password = $PASS" >> $POSTPATH/sql/mysql_virtual_mailbox_limit_maps.cf
echo "hosts = localhost" >> $POSTPATH/sql/mysql_virtual_mailbox_limit_maps.cf
echo "dbname = postfixadmin" >> $POSTPATH/sql/mysql_virtual_mailbox_limit_maps.cf
echo "query = SELECT quota FROM mailbox WHERE username='%s' AND active = '1'" >> $POSTPATH/sql/mysql_virtual_mailbox_limit_maps.cf
echo "user = postfixadmin"  >> $POSTPATH/sql/mysql_virtual_mailbox_maps.cf
echo "password = $PASS" >> $POSTPATH/sql/mysql_virtual_mailbox_maps.cf
echo "hosts = localhost"  >> $POSTPATH/sql/mysql_virtual_mailbox_maps.cf
echo "dbname = postfixadmin" >> $POSTPATH/sql/mysql_virtual_mailbox_maps.cf
echo "query = SELECT maildir FROM mailbox WHERE username='%s' AND active = '1'" >> $POSTPATH/sql/mysql_virtual_mailbox_maps.cf
postconf -e "virtual_mailbox_domains = proxy:mysql:/etc/postfix/sql/mysql_virtual_domains_maps.cf"
postconf -e "virtual_alias_maps = proxy:mysql:/etc/postfix/sql/mysql_virtual_alias_maps.cf, proxy:mysql:/etc/postfix/sql/mysql_virtual_alias_domain_maps.cf, proxy:mysql:/etc/postfix/sql/mysql_virtual_alias_domain_catchall_maps.cf"
postconf -e "virtual_mailbox_maps = proxy:mysql:/etc/postfix/sql/mysql_virtual_mailbox_maps.cf, proxy:mysql:/etc/postfix/sql/mysql_virtual_alias_domain_mailbox_maps.cf"
postconf -e "smtpd_tls_cert_file = /etc/letsencrypt/live/server.$domain/cert.pem"
postconf -e "smtpd_tls_key_file = /etc/letsencrypt/live/server.$domain/privkey.pem"
postconf -e "smtpd_use_tls = yes"
postconf -e "smtpd_tls_auth_only = yes"
postconf -e "smtpd_sasl_type = dovecot"
postconf -e "smtpd_sasl_path = private/auth"
postconf -e "smtpd_sasl_auth_enable = yes"
postconf -e "smtpd_recipient_restrictions = permit_sasl_authenticated, permit_mynetworks, reject_unauth_destination"
postconf -e "mydestination = localhost"
postconf -e "mynetworks = 127.0.0.0/8"
postconf -e "inet_protocols = ipv4"
postconf -e "inet_interfaces = all"
postconf -e "virtual_transport = lmtp:unix:private/dovecot-lmtp"
 
echo "Enabling TLS Postfix encrypt host-to-host"
postconf -e "smtp_tls_security_level = may"
postconf -e "smtp_tls_loglevel=1"
postconf -e "smtp_tls_CAfile = /etc/letsencrypt/live/server.$domain/cert.pem"
echo "submission inet n - n - - smtpd -o syslog_name=postfix/submission -o smtpd_tls_security_level=encrypt -o smtpd_sasl_auth_enable=yes -o smtpd_relay_restrictions=permit_sasl_authenticated,reject -o milter_macro_daemon_name=ORIGINATING" >> $POSTPATH/master.cf 
echo "smtps inet n - n - - smtpd -o syslog_name=postfix/smtps -o smtpd_sasl_auth_enable=yes -o smtpd_relay_restrictions=permit_sasl_authenticated,reject -o milter_macro_daemon_name=ORIGINATING" >> $POSTPATH/master.cf
systemctl restart postfix

echo "Installing Dovecot"
yum -y install dovecot dovecot-mysql > /dev/null

echo "Configuring Dovecot"
cp -f /etc/dovecot/conf.d/10-mail.conf  /etc/dovecot/conf.d/10-mail.conf.res
cp -f /etc/dovecot/conf.d/10-master.conf /etc/dovecot/conf.d/10-master.conf.res
cp -f /etc/dovecot/conf.d/15-lda.conf /etc/dovecot/conf.d/15-lda.conf.res
cp -f /etc/dovecot/conf.d/10-auth.conf /etc/dovecot/conf.d/10-auth.conf.res
echo "mail_location = maildir:/var/vmail/%d/%n" >> /etc/dovecot/conf.d/10-mail.conf
echo "mail_privileged_group = mail" >> /etc/dovecot/conf.d/10-mail.conf
echo "mail_uid = vmail" >> /etc/dovecot/conf.d/10-mail.conf
echo "mail_gid = mail" >> /etc/dovecot/conf.d/10-mail.conf
lineNum="$(grep -n "first_valid_uid ="  /etc/dovecot/conf.d/10-mail.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/dovecot/conf.d/10-mail.conf 
sed -i "${lineNum}i first_valid_uid = 150" /etc/dovecot/conf.d/10-mail.conf
sed -i "${lineNum}i last_valid_uid = 150" /etc/dovecot/conf.d/10-mail.conf
sed -i "s/auth_mechanisms = plain/auth_mechanisms = plain login/g" /etc/dovecot/conf.d/10-auth.conf
sed -i "s/#\!include auth-sql.conf.ext/\!include auth-sql.conf.ext/g" /etc/dovecot/conf.d/10-auth.conf
echo "driver = mysql" >> /etc/dovecot/dovecot-sql.conf.ext
echo "connect = host=localhost dbname=postfixadmin user=postfixadmin password=$PASS" >> /etc/dovecot/dovecot-sql.conf.ext
echo "default_pass_scheme = CRAM-MD5" >> /etc/dovecot/dovecot-sql.conf.ext
echo "password_query = SELECT username as user, password, '/var/vmail/%d/%n' as userdb_home, 'maildir:/var/vmail/%d/%n' as userdb_mail, 150 as userdb_uid, 8 as userdb_gid FROM mailbox WHERE username = '%u' AND active = '1'" >> /etc/dovecot/dovecot-sql.conf.ext
echo "user_query = SELECT '/var/vmail/%d/%u' as home, 'maildir:/var/vmail/%d/%u' as mail, 150 AS uid, 8 AS gid, concat('dirsize:storage=', quota) AS quota FROM mailbox WHERE username = '%u' AND active = '1'" >> /etc/dovecot/dovecot-sql.conf.ext
lineNum="$(grep -n "ssl =" /etc/dovecot/conf.d/10-ssl.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/dovecot/conf.d/10-ssl.conf
sed -i "${lineNum}i ssl = yes " /etc/dovecot/conf.d/10-ssl.conf
cp -f /etc/pki/dovecot/certs/dovecot.pem /etc/pki/dovecot/certs/dovecot.pem.res 2>/dev/null
cp -f /etc/pki/dovecot/private/dovecot.pem /etc/pki/dovecot/private/dovecot.pem.res 2>/dev/null
rm -f /etc/pki/dovecot/certs/dovecot.pem  2>/dev/null
rm -f /etc/pki/dovecot/private/dovecot.pem 2>/dev/null
ln -s /etc/letsencrypt/live/server.$domain/cert.pem  /etc/pki/dovecot/certs/dovecot.pem
ln -s /etc/letsencrypt/live/server.$domain/privkey.pem /etc/pki/dovecot/private/dovecot.pem
echo "postmaster_address = postmaster@$domain" >> /etc/dovecot/conf.d/15-lda.conf
lineNum="$(grep -n "unix_listener lmtp"  /etc/dovecot/conf.d/10-master.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" /etc/dovecot/conf.d/10-master.conf
sed -i "${lineNum}i unix_listener /var/spool/postfix/private/dovecot-lmtp {" /etc/dovecot/conf.d/10-master.conf
((lineNum++))
sed -i "${lineNum}i mode = 0666" /etc/dovecot/conf.d/10-master.conf
((lineNum++))
sed -i "${lineNum}i user = postfix" /etc/dovecot/conf.d/10-master.conf
((lineNum++))
sed -i "${lineNum}i group = postfix" /etc/dovecot/conf.d/10-master.conf
lineNum="$(grep -n "service auth"  /etc/dovecot/conf.d/10-master.conf | head -n 1 | cut -d: -f1)"
((lineNum++))
sed -i "${lineNum}i unix_listener /var/spool/postfix/private/auth {" /etc/dovecot/conf.d/10-master.conf
((lineNum++))
sed -i "${lineNum}i mode = 0666" /etc/dovecot/conf.d/10-master.conf
((lineNum++))
sed -i "${lineNum}i user = postfix" /etc/dovecot/conf.d/10-master.conf
((lineNum++))
sed -i "${lineNum}i group = postfix" /etc/dovecot/conf.d/10-master.conf
((lineNum++))
sed -i "${lineNum}i }" /etc/dovecot/conf.d/10-master.conf
lineNum="$(grep -n "unix_listener auth-userdb"  /etc/dovecot/conf.d/10-master.conf | head -n 1 | cut -d: -f1)"
((lineNum++))
sed -i "${lineNum}i mode = 0660" /etc/dovecot/conf.d/10-master.conf
((lineNum++))
sed -i "${lineNum}i user = vmail" /etc/dovecot/conf.d/10-master.conf
lineNum=$((lineNum + 6))
sed -i "${lineNum}i user = dovecot" /etc/dovecot/conf.d/10-master.conf
lineNum="$(grep -n "service auth-worker"  /etc/dovecot/conf.d/10-master.conf | head -n 1 | cut -d: -f1)"
((lineNum++))
sed -i "${lineNum}i user = vmail" /etc/dovecot/conf.d/10-master.conf
echo "service stats  {" >> /etc/dovecot/dovecot.conf
echo " unix_listener stats-reader {" >> /etc/dovecot/dovecot.conf
echo "  user = vmail" >> /etc/dovecot/dovecot.conf
echo "  group = mail" >> /etc/dovecot/dovecot.conf
echo "mode = 0666" >> /etc/dovecot/dovecot.conf
echo "}" >> /etc/dovecot/dovecot.conf
echo "unix_listener stats-writer {" >> /etc/dovecot/dovecot.conf
echo "  user = vmail" >> /etc/dovecot/dovecot.conf
echo "  group = mail" >> /etc/dovecot/dovecot.conf
echo "mode = 0666" >> /etc/dovecot/dovecot.conf
echo "}" >> /etc/dovecot/dovecot.conf
echo "}" >> /etc/dovecot/dovecot.conf
chown -R vmail:dovecot /etc/dovecot
chmod -R 775 /etc/dovecot
systemctl enable dovecot
systemctl restart dovecot

echo "Creating database of Roundcube"
mysql -u root --password="" -e "CREATE DATABASE roundcube; GRANT ALL PRIVILEGES ON roundcube.* TO 'roundcube_admin'@'localhost' IDENTIFIED BY '${PASS}'; FLUSH PRIVILEGES;"
VER="1.4.11"

echo "Installing Roundcube software"
tar -xzf files/roundcubemail.tar.gz -C $HTTPROOT/server.$domain
mv $HTTPROOT/server.$domain/roundcubemail-$VER  $HTTPROOT/server.$domain/roundcube 
mysql -u root --password="" roundcube <  $HTTPROOT/server.$domain/roundcube/SQL/mysql.initial.sql
cp files/config.inc.php-roundcube  $HTTPROOT/server.$domain/roundcube/config/config.inc.php
sed -i "s/temporal/$PASS/g"  $HTTPROOT/server.$domain/roundcube/config/config.inc.php
rm -rf  $HTTPROOT/server.$domain/roundcube/installer

echo "Extracting files from admin.mail"
tar -xzf files/adminmail.tar.gz -C $HTTPROOT/server.$domain
PA="$HTTPROOT/server.$domain/admin.mail"
chown -R apache:apache $PA
   
echo "Making configuration on admin.mail"
lineNum="$(grep -n "DB_HOST"  $PA/App/Config.php |grep "false" | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" $PA/App/Config.php
sed -i "${lineNum}i const DB_HOST = '127.0.0.1';" $PA/App/Config.php
lineNum="$(grep -n "DB_USER"  $PA/App/Config.php |grep "false" | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" $PA/App/Config.php
sed -i "${lineNum}i const DB_USER = 'postfixadmin';" $PA/App/Config.php
lineNum="$(grep -n "DB_PASSWORD"  $PA/App/Config.php |grep "false" | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" $PA/App/Config.php
sed -i "${lineNum}i const DB_PASSWORD = '$PASS';" $PA/App/Config.php
lineNum="$(grep -n "DB_NAME"  $PA/App/Config.php |grep "false" | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" $PA/App/Config.php
sed -i "${lineNum}i const DB_NAME = 'postfixadmin';" $PA/App/Config.php
lineNum="$(grep -n "DOMAIN"  $PA/App/Config.php |grep "false" | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" $PA/App/Config.php
sed -i "${lineNum}i \const DOMAIN = '${domain}';" $PA/App/Config.php
lineNum="$(grep -n "SITEURL"  $PA/App/Config.php |grep "false" | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" $PA/App/Config.php
sed -i "${lineNum}i \const SITEURL = 'https://server.${domain}/admin.mail/';" $PA/App/Config.php



echo "Adding Alias and directory permission to Http"
lineNum="$(grep -n "/VirtualHost" $HTTPCONF/sites-available/server.$domain.conf | head -n 1 | cut -d: -f1)"
sed -i "${lineNum}d" $HTTPCONF/sites-available/server.$domain.conf
echo "Alias /admin.mail/ \"$HTTPROOT/server.$domain/admin.mail/\"" >> $HTTPCONF/sites-available/server.$domain.conf
echo "<Directory \"$HTTPROOT/server.$domain/admin.mail/\">" >> $HTTPCONF/sites-available/server.$domain.conf
echo "Options Indexes FollowSymLinks"  >> $HTTPCONF/sites-available/server.$domain.conf
echo "AllowOverride All"  >> $HTTPCONF/sites-available/server.$domain.conf
echo "DirectoryIndex index.php"  >> $HTTPCONF/sites-available/server.$domain.conf  
echo "</Directory>" >> $HTTPCONF/sites-available/server.$domain.conf
echo "</VirtualHost>" >> $HTTPCONF/sites-available/server.$domain.conf
chown -R apache:apache  $HTTPROOT/server.$domain/
systemctl reload httpd.service
sleep 5


echo "Removing Version"
systemctl stop  postfix.service
sed -i 's/options {/options { version "Test DNS SERVER";/' /etc/named.conf
sed -i 's/#Banner none/Banner none/' /etc/ssh/sshd_config
sed -i -e '$aServerSignature Off' /etc/httpd/conf/httpd.conf
sed -i -e '$aServerTokens Prod' /etc/httpd/conf/httpd.conf
sed -i 's/expose_php = On/expose_php = off/' /etc/php.ini
sed -i "s/SoftwareHeader/#SoftwareHeader/g" /etc/opendkim.conf
sed -i -e '$aSoftwareHeader no' /etc/opendkim.conf

systemctl start  postfix.service
systemctl reload  postfix.service

echo "Creating user admin credentials on admin.mail"
curl -k -s https://server.$domain/admin.mail/init/index?domain=$domain >> /dev/null 

echo "---------------------------------------------------------------"
echo "Domain: $domain"
echo "rDNS: server.$domain"
echo ""
echo "SMTP Server: server.$domain"
echo "Port: 587"
echo ""
echo "POP3 Server: server.$domain"
echo "Port: 110, 995"
echo ""
echo "IMAP Server: server.$domain"
echo "Port: 143, 993"
echo ""
echo "Email Administrator URL: https://server.$domain/admin.mail"
echo "Username: admin"
echo "Password: admin"
echo ""
echo "Webmail URL: https://server.$domain/roundcube"
echo ""
echo "Apache Web Path [1]: $HTTPROOT/server.$domain"
echo "Apache Web Path [2]: $HTTPROOT/$domain" 
echo "---------------------------------------------------------------"
echo "Finish, restarting server to enable all services!"
reboot

fi

